// Code by DragonMinded
// Please attribute, nothing more

#include <string.h>
#include "drawable.h"
#include "malloc.h"

DrawableImage::DrawableImage()
{
	// Ensure we never draw if we haven't created
	_initialized = false;
	
	// Ensure proper null pointer
	_pixels = NULL;
}

DrawableImage::~DrawableImage()
{
	// Clear memory if needed
	if(_pixels)
	{
		free(_pixels);
		_pixels = NULL;
	}
}		

void DrawableImage::createImage(int width, int height)
{
	// Set parameters
	_width = width;
	_height = height;
	
	// Allocate room
	_pixels = (u8 *)memalign(32, _width * _height * 4);
	
	// Set to zero's for now
	memset(_pixels, 255, _width * _height * 4);
	
	// Move flush cached memory
	DCFlushRange (_pixels, _width * _height * 4);
	
	// Set sprite as valid
	_initialized = true;
}

void DrawableImage::BindTexture(bool bilinear)
{
	if(!_initialized) return;

	// Set up sprite
	GX_InitTexObj(&_texObj, _pixels, _width, _height, GX_TF_RGBA8, GX_CLAMP, GX_CLAMP, GX_FALSE);
	
	// Disable bilinear if not applicable
	if(!bilinear) GX_InitTexObjLOD(&_texObj, GX_NEAR, GX_NEAR, 0.0f, 0.0f, 0.0f, GX_DISABLE, GX_DISABLE, GX_ANISO_1);
	
	// Set the texture properly
	GX_LoadTexObj(&_texObj, GX_TEXMAP0);
}

u32 DrawableImage::getWidth()
{
	return _width;
}

u32 DrawableImage::getHeight()
{
	return _height;
}

void DrawableImage::copyBuffer(u8 *buf)
{
	// This function assumes the u8 buffer is RGBA quads
	u8 *pixBuf = _pixels;
	
	// Loop in chunks of 4
	for(u32 j = 0; j < _height; j += 4)
	{
		for(u32 i = 0; i < _width; i += 4) 
		{
			// First, copy AR chunks
			for(int y = 0; y < 4; y++)
			{
				for(int x = 0; x < 4; x++)
				{
					// Alpha
					*pixBuf++ = buf[(((i + x) + ((j + y) * _width)) * 4) + 3];
					
					// Red
					*pixBuf++ = buf[((i + x) + ((j + y) * _width)) * 4];
				}
			}
			
			// Now, copy GB chunks
			for(int y = 0; y < 4; y++)
			{
				for(int x = 0; x < 4; x++)
				{
					// Green
					*pixBuf++ = buf[(((i + x) + ((j + y) * _width)) * 4) + 1];
					
					// Blue
					*pixBuf++ = buf[(((i + x) + ((j + y) * _width)) * 4) + 2];
				}
			}
		}
	}
	
	// Move flush cached memory
	DCFlushRange (_pixels, _width * _height * 4);
}

Image *DrawableImage::getImage()
{
	return (Image *)this;
}

void DrawableImage::setFont(const char *f)
{
	font.setFont(f);
}

void DrawableImage::setColor(Color c)
{
	font.setColor(c);
}

void DrawableImage::setSize(int s)
{
	font.setSize(s);
}

void DrawableImage::renderText(const char *fmt, ...)
{
	// Need to make room for the sprintf'd text
	char *out = (char *)memalign(32, 1024);
	
	// Need temporary blit buffer
	u8 *blitbuf = (u8 *)memalign(32, _width * _height * 4);
	memset(blitbuf, 0, _width * _height * 4);
	
	// Build using sprintf
	va_list marker;
	va_start(marker,fmt);
	vsprintf(out,fmt,marker);
	va_end(marker);
	
	// Set up buffer
	font.setBuffer(blitbuf, _width, _height);
	
	// Call rendering engine
	font.renderSimple(out);
	
	// Blit to surface
	copyBuffer(blitbuf);
	
	// Free memory
	free(blitbuf);
	free(out);
}
